<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\PharmaviteModel;
use App\Http\ResponseHelper;
use Log;

class PharmaviteController extends Controller
{
    private $responseHelper;
    private $pharmaviteObj;

    public function __construct() {

        $this->responseHelper = new ResponseHelper();
        $this->pharmaviteObj = new PharmaviteModel();
    }

    /**
     * phvContactus post request
     */
    public function phvContactus(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : '';
        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request'
            ];
            $message = 'Failed';
            Log::channel('pharmaviteControllerLog')->info("phvContactus: Invalid endpoint request");
            if (!empty($request->input('callback'))) {
                return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData );
            } else {
                return $this->responseHelper->returnData(null, '600', $message, $returnData );
            }
        }
        Log::channel('pharmaviteControllerLog')->info("phvContactus: Request => ".$requestData);
        $receivedData = json_decode($requestData);
        $requestKeys = ['EMail','FirstName','LastName','MessageBody'];
        $optionalkeys = ['Subject'];
        $keysInTheRequest = [];
        $emptyValueKeys = [];
        foreach ($receivedData as $key => $recValue) {

            $keysInTheRequest[] = $key;
            if (empty($recValue) && !in_array($key, $optionalkeys)) {

                $emptyValueKeys[] = $key;
            }
        }
        $keyComparedStatus = array_diff($requestKeys, $keysInTheRequest);
        if(!empty($keyComparedStatus)) {

            $missedParameters = implode(",",$keyComparedStatus);
            $message = 'Failed';
            $returnData = [
                'error' => $missedParameters.' endpoint parameters are missing in the request'
            ];
            Log::channel('pharmaviteControllerLog')->info("phvContactus:". json_encode($returnData));
            return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData);
        }
        if (!empty($emptyValueKeys)) {

            $emptyValueKeys = implode(",",$emptyValueKeys);
            $message = 'Failed';
            $returnData = [
                'error' => $emptyValueKeys.' endpoint parameter values are empty in the request'
            ];
            Log::channel('pharmaviteControllerLog')->info("phvContactus:". json_encode($returnData));
            return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData);
        }
        $postPath = env('POST_PHVCONTACT_URL',true);
        $response = $this->pharmaviteObj->postRequestData($postPath, $requestData);
        if (strtolower(trim($response)) != 'success') {

            $message = 'Failed';
            $returnData = [
                'error' => 'An error occurred while posting the request to the API',
                'request' => $requestData
            ];
            Log::channel('pharmaviteControllerLog')->info("phvContactus: An error occurred while posting the request to the API");
            Log::channel('pharmaviteControllerLog')->info("phvContactus: Response => ". json_encode($response));
            return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData);
        }
        $message = 'Success';
        $returnData = [
            'message' => 'Successfully posted the given request in the API (Email: '.$receivedData->EMail.')'
        ];
        Log::info($returnData);
        Log::channel('pharmaviteControllerLog')->info("phvContactus: Successfully posted the given request in the API (Email: ".$receivedData->EMail.")");
        return $this->responseHelper->returnData($request->input('callback'), '200', $message, $returnData);
    } 

    /**
     * nmmbg post request
     */
    public function nmmbg(Request $request) {

        $requestData = $request->input('data');
        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request'
            ];
            $message = 'Failed';
            Log::channel('pharmaviteControllerLog')->info("nmmbg: Invalid endpoint request");
            if (!empty($request->input('callback'))) {
                return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData );
            } else {
                return $this->responseHelper->returnData(null, '600', $message, $returnData );
            }
        }
        Log::channel('pharmaviteControllerLog')->info("nmmbg: Request => ".$requestData);
        $receivedData = json_decode($requestData);
        $requestKeys = ['EMail','FirstName','LastName','AddressLine1','City','State','Zip','ReturnProductId','ProductListNumber'];
        $keysInTheRequest = [];
        $emptyValueKeys = [];
        foreach ($receivedData as $key => $recValue) {

            $keysInTheRequest[] = $key;
            if (empty($recValue) && $key !='AddressLine2') {

                $emptyValueKeys[] = $key;
            }
        }
        $keyComparedStatus = array_diff($requestKeys, $keysInTheRequest);
        if(!empty($keyComparedStatus)) {

            $missedParameters = implode(",",$keyComparedStatus);
            $message = 'Failed';
            $returnData = [
                'error' => $missedParameters.' endpoint parameters are missing in the request'
            ];
            Log::channel('pharmaviteControllerLog')->info("nmmbg:".json_encode($returnData));
            return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData );
        }
        if (!empty($emptyValueKeys)) {

            $emptyValueKeys = implode(",",$emptyValueKeys);
            $message = 'Failed';
            $returnData = [
                'error' => $emptyValueKeys.' endpoint parameter values are empty in the request'
            ];
            Log::channel('pharmaviteControllerLog')->info("nmmbg:".json_encode($returnData));
            return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData );
        }
        $postPath = env('POST_NMMBG_URL',true);
        $response = $this->pharmaviteObj->postRequestData($postPath, $requestData);
        if (strtolower(trim($response)) != 'success') {

            $message = 'Failed';
            $returnData = [
                'error' => 'An error occurred while posting the request to the API',
                'request' => $requestData
            ];
            Log::channel('pharmaviteControllerLog')->info("nmmbg: An error occurred while posting the request to the API");
            Log::channel('pharmaviteControllerLog')->info("nmmbg: Response => ".json_encode($response));
            return $this->responseHelper->returnData($request->input('callback'), '600', $message, $returnData );
        }
        $message = 'Success';
        $returnData = [
            'message' => 'Successfully posted the given request in the API (Email: '.$receivedData->EMail.')'
        ];
        Log::channel('pharmaviteControllerLog')->info("nmmbg: Successfully posted the given request in the API (Email: ".$receivedData->EMail.")");
        return $this->responseHelper->returnData($request->input('callback'), '200', $message, $returnData );
    }
}
